/* $Id: field.c,v 1.16 2006/01/27 00:46:32 ppl Exp $ */

/*
	Code that draw the gaming field
*/

#include "xsfcave.h"

#include "field.h"

#define TILES_WIDTH (15 * 4)

#define COLOR_GREEN "#31FF31"

static void do_tile_gradient(Pixmap pixmap, unsigned int height, char const * const [9]);

Bool
field_create(FieldClass *field, Window parent)
{
	/*
		All temporaries (reuse)
	*/
	XGCValues values;
	Pixmap pixmap;

	unsigned int dummy_uint, tile_height;

	const char * const cave_colors[9] =
	{
		"#000000", "#100000",
		"#200000", "#310000",
		"#410000", "#520000",
		"#620000", "#730000",
		"#830000",
	};

	const char * const wall_colors[9] =
	{
		"#83FF83", "#73FF73",
		"#62FF62", "#52FF52",
		"#41FF41", "#31FF31",
		"#20FF20", "#10FF10",
		"#00FF00",
	};

	field->last_tick = 0;

	/*
		Init the tiles
	*/

	field->tiles_update.ts_x_origin = TILES_WIDTH;

	if (XQueryBestTile(display, parent, TILES_WIDTH, 1, &dummy_uint, &tile_height) == False)
		tile_height = 1;

	/*
		tiles_cave
	*/
	if ( (pixmap = XCreatePixmap(display, root, TILES_WIDTH,
	                                  tile_height, depth)) == None)
	{
		return False;
	}

	do_tile_gradient(pixmap, tile_height, cave_colors);

	values.tile = pixmap;
	values.fill_style = FillTiled;
	values.ts_x_origin = TILES_WIDTH;

	field->tiles_cave = XCreateGC(display, parent, GCTile|GCFillStyle|GCTileStipXOrigin, &values);

	XFreePixmap(display, pixmap);

	/*
		tiles_wall
	*/
	if ( (pixmap = XCreatePixmap(display, parent, TILES_WIDTH,
	                             tile_height, depth)) == None)
	{
		return False;
	}

	do_tile_gradient(pixmap, tile_height, wall_colors);

	values.tile = pixmap;
	values.fill_style = FillTiled;
	values.ts_x_origin = TILES_WIDTH;

	field->tiles_wall = XCreateGC(display, parent, GCTile|GCFillStyle|GCTileStipXOrigin, &values);

	XFreePixmap(display, pixmap);

	/*
		Worm Graphic Context
	*/
	values.foreground = get_color(COLOR_PURPLE);
	field->worm_gc = XCreateGC(display, parent, GCForeground, &values);

	/*
		Obstacle Graphic Context
	*/
	values.fill_style = FillSolid;
	values.foreground = get_color(COLOR_GREEN);
	field->obstacle_gc = XCreateGC(display, parent, GCForeground|GCFillStyle, &values);

	return True;
}

void
field_destroy(FieldClass *field)
{
	XFreeGC(display, field->tiles_cave);
	XFreeGC(display, field->tiles_wall);

	XFreeGC(display, field->worm_gc);
	XFreeGC(display, field->obstacle_gc);
}

/*
	Print the game on a GAME_WIDTH/GAME_HEIGHT drawable
*/
void
field_draw(FieldClass *field, GameClass *game, Drawable d)
{
	const Pixmap pixmap = d;

	/* start-evil (eh, it works!) */
	int * const ts_x = &field->tiles_update.ts_x_origin;

	int foo = *ts_x - ((game->tick_count - field->last_tick) % TILES_WIDTH);

	if (foo > 0)
	{
		*ts_x = foo;
	}
	else if (foo < 0)
	{
		*ts_x = TILES_WIDTH - foo;
	}
	else
	{
		*ts_x = TILES_WIDTH;
	}

	XChangeGC(display, field->tiles_wall, GCTileStipXOrigin, &field->tiles_update);
	XChangeGC(display, field->tiles_cave, GCTileStipXOrigin, &field->tiles_update);

	field->last_tick = game->tick_count;
	/* end-evil */

	/*
		Draw the background (red-ish).
	*/
	XFillRectangle(display, pixmap, field->tiles_cave,
	               0, 0, GAME_WIDTH, GAME_HEIGHT);

	/*
		Draw the obstacles (little green bars).
	*/
	XFillRectangles(display, pixmap, field->obstacle_gc,
	                game->obstacle, game->obstacle_count);

	/*
		Draw both top/bottom walls (green-ish).
	*/
	XFillPolygon(display, pixmap, field->tiles_wall,
	             game->wall_top, game->wall_count,
	             Complex, CoordModeOrigin);

	XFillPolygon(display, pixmap, field->tiles_wall,
	             game->wall_bottom, game->wall_count,
	             Complex, CoordModeOrigin);

	/*
		Draw the worm (purple).
	*/
	XDrawSegments(display, pixmap, field->worm_gc, game->worm, game->worm_count);
}

static void
do_tile_gradient(Pixmap pixmap, unsigned int height, const char * const colors[9])
{
	GC gc;
	XGCValues values;

	int i;

	XSegment segments[2];

	segments[0].y1 = 0;
	segments[0].y2 = height;

	segments[1].y1 = 0;
	segments[1].y2 = height;

	values.foreground = get_color(colors[0]);
	values.line_width = 4;

	gc = XCreateGC(display, pixmap, GCForeground|GCLineWidth, &values);

	for (i = 0; i < 8; ++i)
	{
		segments[0].x1 = segments[0].x2 = (i)      * 4;
		segments[1].x1 = segments[1].x2 = (15 - i) * 4;
		XDrawSegments(display, pixmap, gc, segments, 2);

		/* Free pixel */

		values.foreground = get_color(colors[i+1]);
		XChangeGC(display, gc, GCForeground, &values);
	}

	XDrawLine(display, pixmap, gc, 7*4, 0, 7*4, height);

	/* Free pixel */

	XFreeGC(display, gc);

	values.tile = pixmap;
}

const struct FieldClassPublicMethods Field =
{
	field_create,
	field_destroy,
	field_draw,
};
