/* $Id: xsfcave.h,v 1.31 2006/01/31 01:16:17 ppl Exp $ */

#ifndef HEADER_GAME_H
#define HEADER_GAME_H

typedef struct
{
	unsigned long tick_count;   /* number of pixels passed
	                               so far (1 per tick) */

	struct timeval last_tick;  /* Timestamp of the last tick */

	double worm_y;        /* worm current location      */
	double worm_haccel;   /* worm vertical acceleration */
	double worm_weight;   /* the weight of the worm     */

	unsigned int tunel_height;

	Bool game_over;

	/*
		Worm information

		Horizontal segments
	*/
	XSegment worm[WORM_LENGTH];
	unsigned int worm_count; /* Start at 1 and then peek at
	                            WORM_LENGTH for the rest of
	                            the game. */

	/*
		Wall information

		At least 4 points must be present. The first 2
		(index 0 and 1) are reserved for graphic operations.
		Points 3 and 4 (index 2 and 3) _must_ have a
		coordinate in x set to 0 and game width resepectivly.

		Game engine points start at offset 2.

		Those points are use with XFillPolygon

		In X axis, top/bottom are the same
	*/
	XPoint wall_top[250+2];     /* the cave walls */
	XPoint wall_bottom[250+2];  /* the cave walls */
	unsigned int wall_count;    /* total number of elements */
	unsigned int wall_height;   /* rm -f this var probably */

	/*
		Obstacles information
	*/
	XRectangle obstacle[250];
	unsigned int obstacle_count;
	unsigned int obstacle_milestone;
} GameClass;

extern const struct GameClassPublicMethods
{
	void         (*create      )(GameClass *);
	void         (*destroy     )(GameClass *);
	unsigned int (*next_tick_in)(const GameClass *);
	unsigned     (*score       )(const GameClass *);
	unsigned     (*score_replay)(const ReplayClass *);
	void         (*update_ticks)(GameClass *, Bool want_to_go_upwards, ReplayClass *);
} Game;

#endif
