/* $Id: intro.c,v 1.7 2006/01/27 00:46:32 ppl Exp $ */

/*
	Little animation before starting the game. This is only
	graphical eye-candy and not much usefull.
*/

#include "xsfcave.h"

/*
	The x,y coordinates of circle center
*/
#define CIRCLE_X 65
#define CIRCLE_Y 70

#define DIAMETER_MIN 40
#define DIAMETER_MAX 120

#define TITLE "XSFCave"
#define CLICK "Click to start!"

#define FONT_STRING "-misc-fixed-*-*-*-*-20-*-*-*-*-*-*-*"

void
intro_draw_frame(Drawable d)
{
	static unsigned long frame_count;

	static int diameter = DIAMETER_MIN;
	static int diameter_val = 1;

	char buf[1024];    /* huge buffer to be safe */

	GC gc;
	XGCValues values;

	XFontStruct *font;

	const char * const colors[] =
	{
		"#000083", "#000483",
		"#000883", "#000C83",

		"#001083", "#001483",
		"#001883", "#001C83",

		"#002083", "#002483",
		"#002883", "#002C83",

		"#003083", "#003483",
		"#003883", "#003C83",

		"#003C83", "#003883",    /* same thing in reverse */
		"#003483", "#003083",

		"#002C83", "#002883",
		"#002483", "#002083",

		"#001C83", "#001883",
		"#001483", "#001083",

		"#000C83", "#000883",
		"#000483", "#000083",
	};
	const size_t colors_count = sizeof colors / sizeof colors[0];

	if (frame_count == colors_count)
	{
		frame_count = 0;
	}
	else
	{
		frame_count += 1;
	}

	if (diameter > DIAMETER_MAX)
	{
		diameter_val = -1;
	}
	else if (diameter < DIAMETER_MIN)
	{
		diameter_val = 1;
	}

	diameter += diameter_val;

	/* draw the circle animation */
	values.fill_style = FillSolid;
	values.foreground = get_color(colors[frame_count % colors_count]);

	gc = XCreateGC(display, d, GCForeground|GCFillStyle, &values);

	XFillArc(display, d, gc, CIRCLE_X - diameter/2,
	                         CIRCLE_Y - diameter/2,
	         diameter, diameter, 360*64, 360*64);

	/*
		Draw text
	*/
	if ( (font = XLoadQueryFont(display, FONT_STRING)) == NULL)
	{
		puts("Fixed font not available!");
		exit(EXIT_FAILURE);
	}

	values.foreground = BlackPixel(display, screen);
	values.font = font->fid;
	XChangeGC(display, gc, GCForeground|GCFont, &values);

	XDrawString(display, d, gc, 19, 53, TITLE, sizeof TITLE - 1);

	values.foreground = WhitePixel(display, screen);
	XChangeGC(display, gc, GCForeground|GCFont, &values);

	XDrawString(display, d, gc, 17, 50, TITLE, sizeof TITLE - 1);

	if ( (font = XLoadQueryFont(display, "fixed")) == NULL)
	{
		puts("Fixed font not available!");
		exit(EXIT_FAILURE);
	}

	values.font = font->fid;
	XChangeGC(display, gc, GCFont, &values);

	/*
		Click here!
	*/
	values.foreground = get_color("#FF0000"); /* red */

	XChangeGC(display, gc, GCForeground, &values);

	XDrawString(display, d, gc, 20, 65, CLICK, sizeof CLICK - 1);

	/*
		Score and high score
	*/
	values.foreground = WhitePixel(display, screen);
	XChangeGC(display, gc, GCForeground, &values);

	sprintf(buf, "Score      :%5d", score);
	XDrawString(display, d, gc, 10, 100, buf, strlen(buf));

	sprintf(buf, "High Score :%5d", high_score);
	XDrawString(display, d, gc, 10, 120, buf, strlen(buf));

	XFreeGC(display, gc);
}
