/* $Id: replay.c,v 1.3 2006/01/31 01:16:17 ppl Exp $ */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <X11/Xlib.h>

#include "replay.h"

static void  fclose_replay(FILE *);
static FILE *fopen_replay (const char *filename);

static void
add_tick(ReplayClass *replay, Bool want_to_go_upwards)
{
	/*
		Print a warning
	*/
	if (replay->tick_count == sizeof replay->user_updown - 1)
	{
		fputs("xsfcave: replay buffer overflow\n", stderr);
	}

	/*
		Prevent overflow
	*/
	if (replay->tick_count == sizeof replay->user_updown)
	{
		return;
	}

	replay->user_updown[replay->tick_count++] = want_to_go_upwards == True ? 'U' : 'D';
	replay->user_updown[replay->tick_count] = '\0';
}

static void
create(ReplayClass *replay, long seed)
{
	replay->seed = seed;
	replay->tick_count = 0;
	replay->user_updown[0] = '\0';
}

static void
destroy(ReplayClass *replay)
{

}

static Bool
load(ReplayClass *replay, FILE *fp)
{
	int i;
	char xsfcave[] = "xsfcave";
	char version[] = "XX.XX";

	if (fscanf(fp, "%7s %5s %ld %u %u", xsfcave, version, &replay->seed, &replay->tick_count, &replay->score) != 5)
	{
		return False;
	}

	if (strcmp(xsfcave, "xsfcave") != 0)
	{
		return False;
	}

	if (strcmp(version, VERSION) != 0)
	{
		fprintf(stderr, "Cannot play replays from version \"%s\"\n", version);
		return False;
	}

	if (replay->tick_count > sizeof replay->user_updown)
	{
		return False;
	}

	getc(fp);

	if (fread(replay->user_updown, 1, replay->tick_count, fp) != replay->tick_count)
	{
		return False;
	}

	replay->user_updown[replay->tick_count] = '\0';

	for (i = 0; i < replay->tick_count; ++i)
	{
		char ud = replay->user_updown[i];

		if (ud != 'D' && ud != 'U')
		{
			return False;
		}
	}

	return True;
}

/*
	Must be in sync with replay_save_as_string();
*/
static void
save(ReplayClass *replay, FILE *fp)
{
unsigned     game_score_replay(const ReplayClass *);

	fputs("xsfcave " VERSION "\n", fp);
	fprintf(fp, "%ld %u %u\n", replay->seed, replay->tick_count, game_score_replay(replay));

	fwrite(replay->user_updown, 1, replay->tick_count, fp);
}

/*
	Must be in sync with replay_save();
*/
static char *
save_as_string(ReplayClass *replay)
{
unsigned     game_score_replay(const ReplayClass *);
	char *s;

	if ( (s = malloc(500 + replay->tick_count)) == NULL)
	{
		perror("malloc");
		return NULL;
	}

	strcpy(s, "xsfcave " VERSION "\n");
	sprintf(s, "xsfcave " VERSION "\n%ld %u %u\n%s", replay->seed, replay->tick_count, game_score_replay(replay), replay->user_updown);

	return s;
}

static void
save_best(ReplayClass *replay)
{
	FILE *fp;

	if ( (fp = fopen_replay("best.xsfcave")) == NULL)
	{
		return;
	}

	Replay.save(replay, fp);

	fclose_replay(fp);
}

static void
save_best_ever(ReplayClass *replay)
{
	FILE *fp;

#define BESTEVER_FILENAME "best_ever.xsfcave"
	if ( (fp = fopen_replay(BESTEVER_FILENAME)) == NULL)
	{
		return;
	}

	Replay.save(replay, fp);

	fclose_replay(fp);
}

static void
save_last(ReplayClass *replay)
{
	FILE *fp;

	if ( (fp = fopen_replay("last.xsfcave")) == NULL)
	{
		return;
	}

	Replay.save(replay, fp);

	fclose_replay(fp);
}

/*
	Local functions
*/

static void
fclose_replay(FILE *fp)
{
	fclose(fp);
}

static FILE *
fopen_replay(const char *filename)
{
	const char *home;
	const char xsfcave[] = "/.xsfcave/";
	char *fullpath;

	FILE *fp;

	if ( (home = getenv("HOME")) == NULL)
	{
		return NULL;
	}

	if ( (fullpath = malloc(strlen(home) + sizeof xsfcave + strlen(filename))) == NULL)
	{
		perror("malloc");
		return NULL;
	}

	sprintf(fullpath, "%s%s%s", home, xsfcave, filename);

	if ( (fp = fopen(fullpath, "w")) == NULL)
	{
		perror(fullpath);
	}

	free(fullpath);

	return fp;
}

const struct ReplayClassPublicMethods Replay =
{
    add_tick,
    create,
    destroy,
    load,
    save,
    save_as_string,
    save_best,
    save_best_ever,
    save_last,
};
