/* $Id: screenshot.c,v 1.5 2006/01/31 04:19:38 ppl Exp $ */

#include <stdlib.h>

#include <X11/Xlib.h>

#include "xsfcave.h"

#include "screenshot.h"

#define SET_WORD(x, y) do { (x)[0] = ((y) & 0x00ff) >> 0; (x)[1] = ((y) & 0xff00) >> 8; } while (0)
#define SET_DWORD(x, y) do { SET_WORD((x), ((y) & 0x0000ffff) >> 0); SET_WORD((x)+2, ((y) & 0xffff0000) >> 16); } while (0)
#define GET_DWORD(x) ((x)[0] | ((x)[1] << 8) | ((x)[2] << 16) | ((x)[3] << 24))

#define SET_FILEHEADER_TYPE(x, y) do { SET_WORD((x), (y)); } while (0)
#define SET_FILEHEADER_SIZE(x, y) do { SET_DWORD((x)+2, (y)); } while (0)
#define SET_FILEHEADER_RESERVED1(x, y) do { SET_WORD((x)+6, (y)); } while (0)
#define SET_FILEHEADER_RESERVED2(x, y) do { SET_WORD((x)+8, (y)); } while (0)
#define SET_FILEHEADER_BITMAPOFFSET(x, y) do { SET_DWORD((x)+10, (y)); } while (0)

#define GET_FILEHEADER_BITMAPOFFSET(x) GET_DWORD((x)+10)

#define SET_INFOHEADER_SIZE(x, y) do { SET_DWORD((x)+14, (y)); } while (0)
#define SET_INFOHEADER_WIDTH(x, y) do { SET_DWORD((x)+18, (y)); } while (0)
#define SET_INFOHEADER_HEIGHT(x, y) do { SET_DWORD((x)+22, (y)); } while (0)
#define SET_INFOHEADER_PLANES(x, y) do { SET_WORD((x)+26, (y)); } while (0)
#define SET_INFOHEADER_BITCOUNT(x, y) do { SET_WORD((x)+28, (y)); } while (0)
#define SET_INFOHEADER_COMPRESSION(x, y) do { SET_DWORD((x)+30, (y)); } while (0)
#define SET_INFOHEADER_IMAGESIZE(x, y) do { SET_DWORD((x)+34, (y)); } while (0)
#define SET_INFOHEADER_XRESOLUTION(x, y) do { SET_DWORD((x)+38, (y)); } while (0)
#define SET_INFOHEADER_YRESOLUTION(x, y) do { SET_DWORD((x)+42, (y)); } while (0)
#define SET_INFOHEADER_NCOLORS(x, y) do { SET_DWORD((x)+46, (y)); } while (0)
#define SET_INFOHEADER_IMPORTANTCOLORS(x, y) do { SET_DWORD((x)+50, (y)); } while (0)

static void
screenshot_create(ScreenshotClass *ss)
{
	ss->bmp = NULL;
	ss->bmp_size = 0;
}

static void
screenshot_destroy(ScreenshotClass *ss)
{
	free(ss->bmp);
}

static Bool
screenshot_save(ScreenshotClass *ss, FILE *fp)
{
	fwrite(ss->bmp, 1, ss->bmp_size, fp);

	if (ferror(fp))
	{
		return False;
	}

	return True;
}

static Bool
screenshot_capture(ScreenshotClass *ss, Drawable d, unsigned int width, unsigned int height)
{
	XImage *img = NULL;

	unsigned char *bmp = NULL;
	unsigned bmp_filesize = 14 + 40 + height*width*3 + height*(width%4);

	/*
		Using calloc() to zero out the padding bytes.
	*/
	if ( (bmp = calloc(bmp_filesize, 1)) == NULL)
	{
		perror("calloc");
		return False;
	}

	if ( (img = XGetImage(display, d, 0, 0, width, height, AllPlanes, ZPixmap)) == NULL)
	{
		free(bmp);
		return False;
	}

	/*
		File header
	*/
	SET_FILEHEADER_TYPE        (bmp, 0x4d42u     ); /* BM in ASCII */
	SET_FILEHEADER_SIZE        (bmp, bmp_filesize);
	SET_FILEHEADER_RESERVED1   (bmp, 0x0000u     );
	SET_FILEHEADER_RESERVED2   (bmp, 0x0000u     );
	SET_FILEHEADER_BITMAPOFFSET(bmp, 54u         ); /* hardcoded: file+info headers */

	/*
		Info header
	*/
	SET_INFOHEADER_SIZE           (bmp, 40u   ); /* Standard size */
	SET_INFOHEADER_WIDTH          (bmp, width );
	SET_INFOHEADER_HEIGHT         (bmp, height);
	SET_INFOHEADER_PLANES         (bmp, 1u    );
	SET_INFOHEADER_BITCOUNT       (bmp, 24u   );
	SET_INFOHEADER_COMPRESSION    (bmp, 0u    );
	SET_INFOHEADER_IMAGESIZE      (bmp, 0u    );
	SET_INFOHEADER_XRESOLUTION    (bmp, 0u    );
	SET_INFOHEADER_XRESOLUTION    (bmp, 0u    );
	SET_INFOHEADER_NCOLORS        (bmp, 0u    ); /* XXX: is 0 valid here? */
	SET_INFOHEADER_IMPORTANTCOLORS(bmp, 0u    ); /* All colors are important */

	{
		unsigned x, y;

		for (y = 0; y < height; ++y)
		{
			for (x = 0; x < width; ++x)
			{
				unsigned char * const raster = bmp + GET_FILEHEADER_BITMAPOFFSET(bmp);
				unsigned char * const raster_pixel = raster + (x + y*width) * 3 + y*(width%4);

				const unsigned long pixel = XGetPixel(img, x, height - y - 1);

				unsigned char * const r = &raster_pixel[2];
				unsigned char * const g = &raster_pixel[1];
				unsigned char * const b = &raster_pixel[0];

				if (img->depth == 16)
				{
					*r = ((pixel & 0xf800) >> 11) * 256 / 32;
					*g = ((pixel & 0x07e0) >>  5) * 256 / 64;
					*b = ((pixel & 0x001f) >>  0) * 256 / 32;
				}
				else if (img->depth == 24)
				{
					*r = (pixel & 0xff0000) >> 16;
					*g = (pixel & 0x00ff00) >>  8;
					*b = (pixel & 0x0000ff) >>  0;
				}
				else
				{
					free(bmp);
					if (img != NULL)
					{
						XFree(img);
					}
					return False;
				}
			}
		}
	}

	ss->bmp = bmp;
	ss->bmp_size = bmp_filesize;

	if (img != NULL)
	{
		XFree(img);
	}

	return True;
}

const struct ScreenshotClassPublicMethods Screenshot =
{
	screenshot_create,
	screenshot_destroy,
	screenshot_save,
	screenshot_capture,
};
