/* $Id: submit_score.c,v 1.5 2006/01/31 01:16:17 ppl Exp $ */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>

#include <sys/types.h>
#include <sys/socket.h>

#include <netdb.h>
#include <netinet/in.h>

#include "xsfcave.h"
#include "game_engine.h"

#define HOSTNAME "xsfcave.idios.org"
#define HTTP_DESTINATION "/scores/xsfcave-0.5/submit-score.php"
#define BOUNDARY         "AaB03x"

typedef struct sockaddr SA;

static Bool first = True; 

const char post_format_string[] = {
	"POST " HTTP_DESTINATION " HTTP/1.1\r\n"
	"Host: " HOSTNAME "\r\n"
	"Content-Type: multipart/form-data; boundary=" BOUNDARY "\r\n"
	"Content-Length: %d\r\n"
	"\r\n"
};

const char mime_format_string[] = {
	"--" BOUNDARY "\r\n"
	"Content-Disposition: form-data; name=\"nickname\"\r\n"
	"\r\n"
	"%s\r\n"
	"--" BOUNDARY "\r\n"
	"Content-Disposition: form-data; name=\"version\"\r\n"
	"\r\n"
	"%s\r\n"
	"--" BOUNDARY "\r\n"
	"Content-Disposition: form-data; name=\"score\"\r\n"
	"\r\n"
	"%d\r\n"
	"--" BOUNDARY "\r\n"
	"Content-Disposition: form-data; name=\"replay\"; filename=\"last.xsfcave\"\r\n"
	"Content-Type: application/octet-stream\r\n"
	"\r\n"
	"%s\r\n"
	"--" BOUNDARY "\r\n"
	"Content-Disposition: form-data; name=\"screenshot\"; filename=\"screenshot.bmp\"\r\n"
	"Content-Type: image/bmp\r\n"
	"\r\n"
};

#define LAST_LINE "\r\n--" BOUNDARY "--" "\r\n"

/*

*/
void
submit_score(const char *nickname, const char *version, ReplayClass *replay, ScreenshotClass *ss)
{
	int fd;
	pid_t pid;
	struct sockaddr_in addr;
	struct hostent *h;

	char post_data[2500];
	char mime_data[10000];
	ssize_t post_data_length;
	ssize_t mime_data_length;

	ssize_t n;
	char read_buffer[1000];
	char *server_answer;

	switch ( (pid = fork()))
	{
	case 0:    /* Child */
		break;

	case -1:   /* Parent */
		perror("submit-score: fork");
		return;
		break;

	default:
		if (first == True)
		{
			puts("See http://scrap.sf.net/xsfcave/scores/ for the Hall of Fame");
			first = False;
		}
		return;
		break;
	}

	/*
		Dinamicly create the data to be sent
	*/

	sprintf(mime_data, mime_format_string, nickname, version, Game.score_replay(replay), Replay.save_as_string(replay));
	mime_data_length = strlen(mime_data);

	sprintf(post_data, post_format_string, mime_data_length+strlen(LAST_LINE)+ss->bmp_size);
	post_data_length = strlen(post_data);

	/*
		Create socket and connect to HTTP server
	*/

	if ( (fd = socket(PF_INET, SOCK_STREAM, 0)) == -1)
	{
		perror("submit-score: socket");
		_exit(EXIT_FAILURE);
	}

	if ( (h = gethostbyname(HOSTNAME)) == NULL)
	{
		herror("submit-score: gethostbyname");
		_exit(EXIT_FAILURE);
	}

	addr.sin_family = AF_INET;
	addr.sin_port   = htons(80);
	addr.sin_addr   = *( (struct in_addr *) h->h_addr );
	memset(&addr.sin_zero, '\0', sizeof addr.sin_zero);

	if (connect(fd, (SA *)&addr, sizeof addr) == -1)
	{
		perror("submit-score: connect");
		_exit(EXIT_FAILURE);
	}

	/*
		Send POST request to web server.
	*/
	if (write(fd, post_data, post_data_length) < post_data_length)
	{
		fputs("submit-score: socket write error\n", stderr);
		_exit(EXIT_FAILURE);
	}

	if (write(fd, mime_data, mime_data_length) < mime_data_length)
	{
		fputs("submit-score: socket write error\n", stderr);
		_exit(EXIT_FAILURE);
	}

	if (write(fd, ss->bmp, ss->bmp_size) < ss->bmp_size)
	{
		fputs("submit-score: socket write error\n", stderr);
		_exit(EXIT_FAILURE);
	}

	if (write(fd, LAST_LINE, strlen(LAST_LINE)) < strlen(LAST_LINE))
	{
		fputs("submit-score: socket write error\n", stderr);
		_exit(EXIT_FAILURE);
	}

	/*
		Read PHP script reply
	*/
	if ( (n = read(fd, read_buffer, sizeof read_buffer - 1)) > 0)
	{
		read_buffer[n] = '\0';
		if ( (server_answer = strstr(read_buffer, "\r\n\r\n")) != NULL)
		{
			server_answer += sizeof  "\r\n\r\n" - 1;

			if (strcmp(server_answer, "Congratulation!") == 0)
			{
				printf("submit-score: You ('%s') just registred the high score %u\n", nickname, score);
			}
			else
			{
				printf("submit-score: Failed to register the score %u online (%s)\n", score, server_answer);
			}
		}
	}
	else
	{
		if (n == -1)
		{
			perror("submit-score: read");
		}
		printf("submit-score: Failed to registred the score %u online\n", score);
	}

	_exit(EXIT_SUCCESS);
}
