/* $Id: version_check.c,v 1.2 2006/01/31 01:16:17 ppl Exp $ */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>

#include <sys/types.h>
#include <sys/socket.h>

#include <netdb.h>
#include <netinet/in.h>

#define HOSTNAME "xsfcave.idios.org"

typedef struct sockaddr SA;

/*
	Download the document
	"http://xsfcave.idios.org/version.txt" which
	informs us of the latest version.
*/
pid_t
version_check(void)
{
	int fd;
	pid_t pid;
	struct sockaddr_in addr;
	struct hostent *h;
	ssize_t read_count;

	char get_data[] = "GET /version.txt HTTP/1.1\r\nHost: " HOSTNAME "\r\n\r\n";
	char read_buffer[2000];
	char *answer;

	switch ( (pid = fork()))
	{
	case 0:    /* Child */
		break;

	case -1:   /* Parent */
		perror("version-check: fork");
		return (pid_t)-1;
		break;

	default:
		return pid;
		break;
	}

	if ( (fd = socket(PF_INET, SOCK_STREAM, 0)) == -1)
	{
		perror("version-check: socket");
		_exit(EXIT_FAILURE);
	}

	if ( (h = gethostbyname(HOSTNAME)) == NULL)
	{
		herror("version-check: gethostbyname");
		_exit(EXIT_FAILURE);
	}

	addr.sin_family = AF_INET;
	addr.sin_port   = htons(80);
	addr.sin_addr   = *( (struct in_addr *) h->h_addr );
	memset(&addr.sin_zero, '\0', sizeof addr.sin_zero);

	if (connect(fd, (SA *)&addr, sizeof addr) == -1)
	{
		perror("version-check: connect");
		_exit(EXIT_FAILURE);
	}

	/*
		Send GET request to web server.
	*/
	if (write(fd, get_data, sizeof get_data - 1) < sizeof get_data -1)
	{
		fputs("version-check: socket write error\n", stderr);
		_exit(EXIT_FAILURE);
	}

	/*
		Read server reply
	*/
	if ( (read_count = read(fd, read_buffer, sizeof read_buffer - 1)) == -1)
	{
		perror("version-check: read");
		_exit(EXIT_FAILURE);
	}

	read_buffer[read_count] = '\0';

	if ( (answer = strstr(read_buffer, "\r\n\r\n")) == NULL)
	{
		fputs("version-check: invalid server reply\n", stderr);
		_exit(EXIT_FAILURE);
	}

	answer += sizeof  "\r\n\r\n" - 1;

	puts("Your are currently using " PACKAGE " " VERSION);

	/*
		`answer' points to the document.
	*/
	if (strcmp(answer, VERSION "\n") == 0)
	{
		puts("You are running the latest release");
	}
	else
	{
		printf("The latest available version is: %s", answer);
	}

	_exit(EXIT_SUCCESS);
}
